@@ -1,8 +1,8 @@
 /**************************************************************************
  *   color.c  --  This file is part of GNU nano.                          *
  *                                                                        *
- *   Copyright (C) 2001-2011, 2013-2021 Free Software Foundation, Inc.    *
- *   Copyright (C) 2014-2017, 2020 Benno Schulenberg                      *
+ *   Copyright (C) 2001-2011, 2013-2024 Free Software Foundation, Inc.    *
+ *   Copyright (C) 2014-2017, 2020, 2021 Benno Schulenberg                *
  *                                                                        *
  *   GNU nano is free software: you can redistribute it and/or modify     *
  *   it under the terms of the GNU General Public License as published    *
@@ -53,6 +53,7 @@ void set_interface_colorpairs(void)
 			}
 			init_pair(index + 1, combo->fg, combo->bg);
 			interface_color_pair[index] = COLOR_PAIR(index + 1) | combo->attributes;
+			rescind_colors = FALSE;
 		} else {
 			if (index == FUNCTION_TAG || index == SCROLL_BAR)
 				interface_color_pair[index] = A_NORMAL;
@@ -61,7 +62,7 @@ void set_interface_colorpairs(void)
 			else if (index == SPOTLIGHTED) {
 				init_pair(index + 1, COLOR_BLACK, COLOR_YELLOW + (COLORS > 15 ? 8 : 0));
 				interface_color_pair[index] = COLOR_PAIR(index + 1);
-			} else if (index == PROMPT_BAR)
+			} else if (index == MINI_INFOBAR || index == PROMPT_BAR)
 				interface_color_pair[index] = interface_color_pair[TITLE_BAR];
 			else if (index == ERROR_MESSAGE) {
 				init_pair(index + 1, COLOR_WHITE, COLOR_RED);
@@ -72,6 +73,11 @@ void set_interface_colorpairs(void)
 
 		free(color_combo[index]);
 	}
+
+	if (rescind_colors) {
+		interface_color_pair[SPOTLIGHTED] = A_REVERSE;
+		interface_color_pair[ERROR_MESSAGE] = A_REVERSE;
+	}
 }
 
 /* Assign a pair number to each of the foreground/background color combinations
@@ -234,7 +240,7 @@ void check_the_multis(linestruct *line)
 	char *afterstart;
 
 	/* If there is no syntax or no multiline regex, there is nothing to do. */
-	if (openfile->syntax == NULL || openfile->syntax->nmultis == 0)
+	if (!openfile->syntax || openfile->syntax->multiscore == 0)
 		return;
 
 	if (line->multidata == NULL) {
@@ -255,12 +261,14 @@ void check_the_multis(linestruct *line)
 		if (line->multidata[ink->id] == NOTHING) {
 			if (!astart)
 				continue;
-		} else if (line->multidata[ink->id] & (WHOLELINE|WOULDBE)) {
-			if (!astart && !anend)
+		} else if (line->multidata[ink->id] == WHOLELINE) {
+			/* Ensure that a detected start match is not actually an end match. */
+			if (!anend && (!astart || regexec(ink->end, line->data, 1,
+												&endmatch, 0) != 0))
 				continue;
 		} else if (line->multidata[ink->id] == JUSTONTHIS) {
-			if (astart && anend && regexec(ink->start, line->data + endmatch.rm_eo,
-														1, &startmatch, 0) != 0)
+			if (astart && anend && regexec(ink->start, line->data + startmatch.rm_eo +
+											endmatch.rm_eo, 1, &startmatch, 0) != 0)
 				continue;
 		} else if (line->multidata[ink->id] == STARTSHERE) {
 			if (astart && !anend)
@@ -272,6 +280,7 @@ void check_the_multis(linestruct *line)
 
 		/* There is a mismatch, so something changed: repaint. */
 		refresh_needed = TRUE;
+		perturbed = TRUE;
 		return;
 	}
 }
@@ -284,7 +293,7 @@ void precalc_multicolorinfo(void)
 	regmatch_t startmatch, endmatch;
 	linestruct *line, *tailline;
 
-	if (!openfile->syntax || openfile->syntax->nmultis == 0 || ISSET(NO_SYNTAX))
+	if (!openfile->syntax || openfile->syntax->multiscore == 0 || ISSET(NO_SYNTAX))
 		return;
 
 //#define TIMEPRECALC  123
@@ -296,7 +305,7 @@ void precalc_multicolorinfo(void)
 	/* For each line, allocate cache space for the multiline-regex info. */
 	for (line = openfile->filetop; line != NULL; line = line->next)
 		if (!line->multidata)
-			line->multidata = nmalloc(openfile->syntax->nmultis * sizeof(short));
+			line->multidata = nmalloc(openfile->syntax->multiscore * sizeof(short));
 
 	for (ink = openfile->syntax->color; ink != NULL; ink = ink->next) {
 		/* If this is not a multi-line regex, skip it. */
@@ -342,21 +351,17 @@ void precalc_multicolorinfo(void)
 											1, &endmatch, 0) != 0)
 					tailline = tailline->next;
 
-				/* When there is no end match, mark relevant lines as such. */
-				if (tailline == NULL) {
-					for (; line->next != NULL; line = line->next)
-						line->multidata[ink->id] = WOULDBE;
-					line->multidata[ink->id] = WOULDBE;
-					break;
-				}
-
-				/* We found it, we found it, la lala lala.  Mark the lines. */
 				line->multidata[ink->id] = STARTSHERE;
 
 				// Note that this also advances the line in the main loop.
 				for (line = line->next; line != tailline; line = line->next)
 					line->multidata[ink->id] = WHOLELINE;
 
+				if (tailline == NULL) {
+					line = openfile->filebot;
+					break;
+				}
+
 				tailline->multidata[ink->id] = ENDSHERE;
 
 				/* Look for a possible new start after the end match. */
